#!/usr/bin/perl
#*****************************************************************************
# 
#  Copyright (c) 2002 Guillaume Cottenceau (gc at mandrakesoft dot com)
#  Copyright (c) 2003 Olivier Blin (oliv.blin at laposte dot net)
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2, as
#  published by the Free Software Foundation.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
#*****************************************************************************

use lib qw(/usr/lib/libDrakX);
use common;

require_root_capability();

use rpmdrake;

eval { require ugtk2; ugtk2->import(qw(:all)) };
if ($@) {
    print "This program cannot be run in console mode.\n";
    c::_exit(0);  
}

sub my_update_sources_interactive {
    my ($urpm, @toupdates) = @_;

    my @entries = map { $_->{name} } @{$urpm->{media}};
    my $w = ugtk2->new(N("Update media"), grab => 1, center => 1);
    my @buttons;
    gtkadd($w->{window},
	   gtkpack__(Gtk2::VBox->new(0,5),
		     Gtk2::Label->new(N("Select the media you wish to update:")),
		     (@buttons = map { Gtk2::CheckButton->new($_->{name}) } @{$urpm->{media}}),
		     Gtk2::HSeparator->new,
		     gtkpack(create_hbox(),
			     gtksignal_connect(Gtk2::Button->new(N("Update")), clicked => sub {
						   $w->{retval} = any { $_->get_active } @buttons;
						   each_index { $_->get_active and $urpm->select_media($urpm->{media}[$::i]{name}) } @buttons;
						   Gtk2->main_quit;
					       }),
			     gtksignal_connect(Gtk2::Button->new(N("Cancel")), clicked => sub { $w->{retval} = 0; Gtk2->main_quit }))));
    each_index { grep { $_ eq $urpm->{media}[$::i]{name} } @toupdates and $_->set_active(true)} @buttons;
    if ($w->main) {
	check_and_update_sources($urpm);
	return 1;
    }
    return 0;
}

sub check_and_update_sources {
    my ($urpm) = @_;

    #- force ignored media to be returned alive.
    my $something_todo = 0;
    foreach (@{$urpm->{media}}) {
	$_->{modified} and delete $_->{ignore};
	$_->{modified} and ++$something_todo;
    }

    $something_todo or die N("the entry to update is missing\n");

    update_sources($urpm);
}

sub main {
    my (@toupdates, $no_auto);
    my $urpm;

    $urpm = urpm->new;
    $urpm->read_config; 

    while ($_ = shift @_) {
	/^--no-auto$/ and $no_auto = 1, next;
	push @toupdates, $_;
    }

    if ($no_auto) {
	my_update_sources_interactive($urpm, @toupdates);
    } else {
	$urpm->select_media(@toupdates);
	check_and_update_sources($urpm);
    }
}

main(@ARGV);

myexit 0;
