#!/usr/bin/env python

import pygtk
pygtk.require('2.0')

import mdk_check_update_common

import gtk
import gnome.ui
import trayicon
import pygspawn

update_media_cmd = "/usr/sbin/g-update-urpm-sources.pl "
prompt_update_media_cmd = update_media_cmd + "--no-auto "

pix_uptodate = None
pix_obsolete = None
status_img = None
refresh_img = None
first_timer = None
menu = None
tray = None

def iso2utf(s):
    return unicode(s, "iso8859-15").encode("utf-8")

def force_gtk_refresh():
	while gtk.events_pending():
		gtk.mainiteration()

def quit_cb(widget, data):	
	gtk.main_quit()

def about_cb(widget, data):
	about = gnome.ui.About(mdk_check_update_common.program_name, mdk_check_update_common.version, "(C) 2003 Olivier Blin", "Urpmi media updates notifier.", ["Olivier Blin"], [])
	about.show()

def update_media_cb(widget, options):
    pygspawn.g_spawn_command_line_async(update_media_cmd + options)

def prompt_update_media_cb(widget, options):
    pygspawn.g_spawn_command_line_async(prompt_update_media_cmd + options)

def update_menu(tray):
	global menu

	menu_items = []

	medium_index = 0
	uptodate_media = 0
	obsolete_media = 0
	for medium in mdk_check_update_common.media:
            img = gtk.Image()
            if medium['type'] in ['ftp','rsync']:
                if 'update_available' in medium:
                    img.set_from_pixbuf(pix_obsolete)
                    obsolete_media += 1
                elif not 'error' in medium:
                    img.set_from_pixbuf(pix_uptodate)
                    uptodate_media += 1
                else:
                    img.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_MENU)
                # end if
            else:
                img.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_MENU)
            # end if
            menu_items.append([medium['name'].replace('_', '__'), img, update_media_cb, medium['name']])
        # end for

	menu_items.append([None, None, None, []])

	img = gtk.Image()
	img.set_from_stock(gtk.STOCK_JUMP_TO, gtk.ICON_SIZE_MENU)
	menu_items.append(["Update new media", img, prompt_update_media_cb, mdk_check_update_common.list_obsolete_media()])

	img = gtk.Image()
	img.set_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_MENU)
	menu_items.append(["Refresh", img, update_status_cb, tray])

	menu_items.append([None, None, None, []])

	img = gtk.Image()
	img.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_MENU)
	menu_items.append(["About", img, about_cb, []])

	img = gtk.Image()
	img.set_from_stock(gtk.STOCK_QUIT, gtk.ICON_SIZE_MENU)
	menu_items.append(["Quit", img, quit_cb, []])
	
	menu = gtk.Menu()
	for label, img, cb, data in menu_items:
		if img:
			item = gtk.ImageMenuItem(label)
			item.set_image(img)
		elif label:
			item = gtk.MenuItem(label)
		else:
			item = gtk.SeparatorMenuItem()
		# end if
		if cb:
			item.connect("activate", cb, data)
		# end if
		menu.add(item)
	menu.show_all()

	if obsolete_media > 0:
		status_img.set_from_pixbuf(pix_obsolete)
	elif uptodate_media > 0:
		status_img.set_from_pixbuf(pix_uptodate)
	else:
		status_img.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_SMALL_TOOLBAR)
	# end if

def start_swap(img1, img2):
	img2.show()
	img1.hide()
	img1.set_data("swap", 1)
	img1.set_data("swap", 1)
	gtk.timeout_add(500, swap_img, status_img, refresh_img)

def stop_swap(img1, img2):
	img1.set_data("swap", 0)
	img1.show()
	img2.hide()

def swap_img(img1, img2):
	if img1.get_data("swap") == 0 or img1.get_data("swap") == 0:
		return gtk.FALSE
	if img1.get_property("visible"):
		img1.hide()
		img2.show()
	else:
		img1.show()
		img2.hide()
	return gtk.TRUE

def start_blink(img):
	img.show()
	img.set_data("blink", 1)
	gtk.timeout_add(500, blink_img, refresh_img)

def stop_blink(img):
	img.set_data("blink", 0)
	img.hide()

def blink_img(img):
	if img.get_data("blink") == 0:
		return gtk.FALSE
	if img.get_property("visible"):
		img.hide()
	else:
		img.show()
	return gtk.TRUE

def show_menu(widget, event):
    menu.popup(None,None,None,event.button,event.time)

def update_status_cb(widget, tray):
    update_status(tray)

def update_status(tray):
	start_swap(status_img, refresh_img)
	force_gtk_refresh()
	mdk_check_update_common.read_config()
	mdk_check_update_common.check_media()
	update_menu(tray)
	stop_swap(status_img, refresh_img)
	return gtk.TRUE

def first_status_update(tray):
	global first_timer
	tray.send_message(1000, "First update")
	update_status(tray)
	gtk.timeout_remove(first_timer)
	gtk.timeout_add(mdk_check_update_common.update_interval*1000, update_status, tray)
	return gtk.FALSE

def init_pixs():
	global pix_uptodate, pix_obsolete, refresh_img, status_img
	pix_uptodate = gtk.gdk.pixbuf_new_from_file(mdk_check_update_common.icon_uptodate)
	pix_obsolete = gtk.gdk.pixbuf_new_from_file(mdk_check_update_common.icon_obsolete)
	refresh_img = gtk.Image()
	refresh_img.set_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_SMALL_TOOLBAR)
	status_img = gtk.Image()

def mdk_check_update():
	global first_timer, tray, status_img
	gnome.init(mdk_check_update_common.program_name, mdk_check_update_common.version)
	init_pixs()
	hbox = gtk.HBox()
	hbox.add(status_img)
	hbox.add(refresh_img)
	tray = trayicon.TrayIcon(mdk_check_update_common.program_name)
	tray.add_events(gtk.gdk.BUTTON_PRESS_MASK)
	tray.connect("button_press_event", show_menu)
	tray.add(hbox)
	tray.show_all()
	refresh_img.hide()
	tray.send_message(1000, "Hello!")
	update_menu(tray)
	first_timer = gtk.timeout_add(mdk_check_update_common.first_update*1000, first_status_update, tray)
	gtk.main()

mdk_check_update_common.parse_opts()
mdk_check_update()
